@@ -14,7 +14,7 @@ use tracing::info;
 /// Generate a Slumber request collection from an external format
 ///
 /// See docs for more info on formats:
-/// https://slumber.lucaspickering.me/book/cli/import.html
+/// https://slumber.lucaspickering.me/cli/import.html
 #[derive(Clone, Debug, Parser)]
 pub struct ImportCommand {
     /// Input format
@@ -64,7 +64,8 @@ impl Subcommand for ImportCommand {
                     .open(&output_file)
                     .context(format!(
                         "Error opening collection output file \
-                        {output_file:?}"
+                        `{}`",
+                        output_file.display()
                     ))?,
             ),
             None => Box::new(io::stdout()),
