@@ -33,9 +33,12 @@ impl Subcommand for NewCommand {
             .write(true)
             .truncate(self.overwrite)
             .open(&path)
-            .with_context(|| format!("Error opening file {path:?}"))?;
-        file.write_all(SOURCE)
-            .with_context(|| format!("Error writing to file {path:?}"))?;
+            .with_context(|| {
+                format!("Error opening file `{}`", path.display())
+            })?;
+        file.write_all(SOURCE).with_context(|| {
+            format!("Error writing to file `{}`", path.display())
+        })?;
 
         eprintln!("New collection created at `{}`", path.display());
 
@@ -49,16 +52,17 @@ mod tests {
     use indexmap::indexmap;
     use pretty_assertions::assert_eq;
     use rstest::rstest;
+    use serde_json::json;
     use slumber_core::{
         collection::{
             Chain, ChainSource, Collection, Folder, Profile, Recipe,
             RecipeBody, RecipeNode,
         },
-        http::{HttpMethod, content_type::ContentType},
+        http::HttpMethod,
         test_util::by_id,
     };
     use slumber_util::{Factory, TempDir, temp_dir};
-    use std::{env, fs};
+    use std::fs;
 
     /// Test creating a new collection file, specifying the path in various ways
     #[rstest]
@@ -72,10 +76,9 @@ mod tests {
         #[case] global_file_arg: Option<&str>,
         #[case] expected_created_path: PathBuf,
     ) {
-        // Lock an empty env as a proxy for the cwd. We can't just construct
-        // full paths because the default value will always be relative to cwd
-        let _guard = env_lock::lock_env([] as [(&str, Option<&str>); 0]);
-        env::set_current_dir(&*temp_dir).unwrap();
+        // Lock the cwd. We can't just construct full paths because the default
+        // value will always be relative to cwd
+        let _guard = env_lock::lock_current_dir(&*temp_dir).unwrap();
 
         let command = NewCommand {
             file: file_arg.map(PathBuf::from),
@@ -103,6 +106,7 @@ mod tests {
     fn test_deserialize() {
         let collection: Collection = serde_yaml::from_slice(SOURCE).unwrap();
         let expected = Collection {
+            name: Some("My Collection".into()),
             profiles: by_id([Profile {
                 id: "example".into(),
                 name: Some("Example Profile".into()),
@@ -137,11 +141,12 @@ mod tests {
                         name: Some("Example Request 2".into()),
                         method: HttpMethod::Post,
                         url: "{{host}}/anything".into(),
-                        body: Some(RecipeBody::Raw {
-                            body: "{\n  \"data\": \"{{chains.example}}\"\n}"
-                                .into(),
-                            content_type: Some(ContentType::Json),
-                        }),
+                        body: Some(
+                            RecipeBody::json(
+                                json!({"data": "{{chains.example}}"}),
+                            )
+                            .unwrap(),
+                        ),
                         ..Recipe::factory(())
                     })]),
                 }),
