@@ -1,10 +1,10 @@
 //! Logic related to input handling. This is considered part of the controller.
 
 use anyhow::{anyhow, bail};
-use crossterm::event::{KeyCode, KeyEvent, KeyModifiers, MediaKeyCode};
-use derive_more::Display;
+use derive_more::{Deref, Display};
+use indexmap::{IndexMap, indexmap};
 use itertools::Itertools;
-use serde::{Deserialize, Serialize};
+use serde::{Deserialize, Deserializer, Serialize};
 use slumber_util::Mapping;
 use std::{
     borrow::Cow,
@@ -12,6 +12,7 @@ use std::{
     iter,
     str::FromStr,
 };
+use terminput::{KeyCode, KeyEvent, KeyModifiers, MediaKeyCode};
 
 /// Key code to string mappings
 const KEY_CODES: Mapping<'static, KeyCode> = Mapping::new(&[
@@ -29,7 +30,6 @@ const KEY_CODES: Mapping<'static, KeyCode> = Mapping::new(&[
     (KeyCode::PageUp, &["pageup", "pgup"]),
     (KeyCode::PageDown, &["pagedown", "pgdn"]),
     (KeyCode::Tab, &["tab"]),
-    (KeyCode::BackTab, &["backtab"]),
     (KeyCode::Backspace, &["backspace"]),
     (KeyCode::Delete, &["delete", "del"]),
     (KeyCode::Insert, &["insert", "ins"]),
@@ -76,7 +76,7 @@ const KEY_MODIFIERS: Mapping<'static, KeyModifiers> = Mapping::new(&[
     // vvvvv If making changes, make sure to update the docs vvvvv
     (KeyModifiers::SHIFT, &["shift"]),
     (KeyModifiers::ALT, &["alt"]),
-    (KeyModifiers::CONTROL, &["ctrl"]),
+    (KeyModifiers::CTRL, &["ctrl"]),
     (KeyModifiers::SUPER, &["super"]),
     (KeyModifiers::HYPER, &["hyper"]),
     (KeyModifiers::META, &["meta"]),
@@ -169,6 +169,9 @@ pub enum Action {
     #[display("Help")]
     /// Open the help modal
     OpenHelp,
+    /// Open collection selection modal (unbound by default)
+    #[display("Select Collection")]
+    SelectCollection,
     /// Select profile list pane
     SelectProfileList,
     /// Select recipe list pane
@@ -292,7 +295,6 @@ impl Display for KeyCombination {
 
         // Write base code
         match self.code {
-            KeyCode::BackTab => write!(f, "<shift{}tab>", Self::SEPARATOR),
             KeyCode::Tab => write!(f, "<tab>"),
             KeyCode::Up => write!(f, "↑"),
             KeyCode::Down => write!(f, "↓"),
@@ -331,10 +333,19 @@ impl FromStr for KeyCombination {
         let code = tokens
             .next_back()
             .ok_or_else(|| anyhow!("Empty key combination"))?;
-        let mut code: KeyCode = parse_key_code(code)?;
+        let mut modifiers = KeyModifiers::NONE;
+        // `backtab` is what crossterm calls `shift tab`. We supported it in the
+        // past because this used to map directly to crossterm. Keeping this
+        // mapping for backward compatibility. We need snowflake logic because
+        // it's a code that maps to a code+modifier
+        let code: KeyCode = if code == "backtab" {
+            modifiers |= KeyModifiers::SHIFT;
+            KeyCode::Tab
+        } else {
+            parse_key_code(code)?
+        };
 
         // Parse modifiers, left-to-right
-        let mut modifiers = KeyModifiers::NONE;
         for modifier in tokens {
             let modifier = parse_key_modifier(modifier)?;
             // Prevent duplicate
@@ -344,13 +355,6 @@ impl FromStr for KeyCombination {
             modifiers |= modifier;
         }
 
-        // Special case - crossterm treats shift+tab as backtab, translate it
-        // automatically for the user
-        if code == KeyCode::Tab && modifiers.contains(KeyModifiers::SHIFT) {
-            code = KeyCode::BackTab;
-            modifiers -= KeyModifiers::SHIFT;
-        }
-
         Ok(Self { code, modifiers })
     }
 }
@@ -376,6 +380,101 @@ impl TryFrom<String> for KeyCombination {
     }
 }
 
+/// Mapping of actions to input bindings
+///
+/// Intuitively this should be binding:action since we get key events from the
+/// user and need to look up the corresponding actions. But we can't look up a
+/// binding from the map based on an input event because event<=>binding
+/// matching is more nuanced that simple equality (e.g. bonus modifiers keys can
+/// be ignored). We have to iterate over map when checking inputs, but keying by
+/// action at least allows us to look up action=>binding for help text.
+#[derive(Clone, Debug, Deref, Serialize)]
+#[cfg_attr(test, derive(PartialEq))]
+#[serde(transparent)]
+pub struct InputMap(IndexMap<Action, InputBinding>);
+
+impl InputMap {
+    fn new(user_bindings: IndexMap<Action, InputBinding>) -> Self {
+        let mut new = Self::default();
+        // User bindings should overwrite any default ones
+        new.0.extend(user_bindings);
+        // If the user overwrote an action with an empty binding, remove it from
+        // the map. This has to be done *after* the extend, so the default
+        // binding is also dropped
+        new.0.retain(|_, binding| !binding.is_empty());
+        new
+    }
+}
+
+impl Default for InputMap {
+    /// Default input bindings
+    fn default() -> Self {
+        Self(indexmap! {
+            // vvvvv If making changes, make sure to update the docs vvvvv
+            Action::Quit => KeyCode::Char('q').into(),
+            Action::ForceQuit => KeyCombination {
+                code: KeyCode::Char('c'),
+                modifiers: KeyModifiers::CTRL,
+            }.into(),
+            Action::ScrollLeft => KeyCombination {
+                code: KeyCode::Left,
+                modifiers: KeyModifiers::SHIFT,
+            }.into(),
+            Action::ScrollRight => KeyCombination {
+                code: KeyCode::Right,
+                modifiers: KeyModifiers::SHIFT,
+            }.into(),
+            Action::OpenActions => KeyCode::Char('x').into(),
+            Action::OpenHelp => KeyCode::Char('?').into(),
+            Action::Fullscreen => KeyCode::Char('f').into(),
+            Action::ReloadCollection => KeyCode::F(5).into(),
+            Action::History => KeyCode::Char('h').into(),
+            Action::Search => KeyCode::Char('/').into(),
+            Action::Export => KeyCode::Char(':').into(),
+            Action::PreviousPane => KeyCombination {
+                code: KeyCode::Tab,
+                modifiers: KeyModifiers::SHIFT,
+            }.into(),
+            Action::NextPane => KeyCode::Tab.into(),
+            Action::Up => KeyCode::Up.into(),
+            Action::Down => KeyCode::Down.into(),
+            Action::Left => KeyCode::Left.into(),
+            Action::Right => KeyCode::Right.into(),
+            Action::PageUp => KeyCode::PageUp.into(),
+            Action::PageDown => KeyCode::PageDown.into(),
+            Action::Home => KeyCode::Home.into(),
+            Action::End => KeyCode::End.into(),
+            Action::Submit => KeyCode::Enter.into(),
+            Action::Toggle => KeyCode::Char(' ').into(),
+            Action::Cancel => KeyCode::Esc.into(),
+            Action::Delete => KeyCode::Delete.into(),
+            Action::Edit => KeyCode::Char('e').into(),
+            Action::Reset => KeyCode::Char('z').into(),
+            Action::View => KeyCode::Char('v').into(),
+            Action::SelectCollection => KeyCode::F(3).into(),
+            Action::SelectProfileList => KeyCode::Char('p').into(),
+            Action::SelectRecipeList => KeyCode::Char('l').into(),
+            Action::SelectRecipe => KeyCode::Char('c').into(),
+            Action::SelectResponse => KeyCode::Char('r').into(),
+            // ^^^^^ If making changes, make sure to update the docs ^^^^^
+        })
+    }
+}
+
+impl<'de> Deserialize<'de> for InputMap {
+    /// Deserialize an input map. First we deserialize the user's provided
+    /// bindings, then we'll populate the map with the defaults so the consumer
+    /// has access to all the bindings in one place
+    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
+    where
+        D: Deserializer<'de>,
+    {
+        let user_bindings: IndexMap<Action, InputBinding> =
+            IndexMap::deserialize(deserializer)?;
+        Ok(Self::new(user_bindings))
+    }
+}
+
 /// Parse a plain key code
 fn parse_key_code(s: &str) -> anyhow::Result<KeyCode> {
     // Check for plain char code
@@ -419,23 +518,15 @@ fn stringify_key_modifier(modifier: KeyModifiers) -> Cow<'static, str> {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crossterm::event::{KeyEventKind, KeyEventState, MediaKeyCode};
     use rstest::rstest;
     use serde_test::{Token, assert_de_tokens, assert_de_tokens_error};
     use slumber_util::assert_err;
+    use terminput::{KeyEventKind, KeyEventState};
 
     #[rstest]
     #[case::whitespace_stripped(" w ", KeyCode::Char('w'))]
     #[case::f_key("f2", KeyCode::F(2))]
     #[case::tab("tab", KeyCode::Tab)]
-    #[case::backtab("backtab", KeyCode::BackTab)]
-    // crossterm treats shift+tab as a special case, we translate for
-    // convenience
-    #[case::shift_tab("shift tab", KeyCode::BackTab)]
-    #[case::multiple_modifiers("alt shift tab", KeyCombination {
-        code: KeyCode::BackTab,
-        modifiers: KeyModifiers::ALT
-    })]
     #[case::page_up("pgup", KeyCode::PageUp)]
     #[case::page_down("pgdn", KeyCode::PageDown)]
     #[case::caps_lock("capslock", KeyCode::CapsLock)]
@@ -456,6 +547,17 @@ mod tests {
         code: KeyCode::F(2),
         modifiers: KeyModifiers::all(),
     })]
+    // Backward compatibility: crossterm translates shift+tab as a separate
+    // keycode call backtab. We previously used crossterm directly in this crate
+    // so we supported this
+    #[case::backtab("backtab", KeyCombination {
+        code: KeyCode::Tab,
+        modifiers: KeyModifiers::SHIFT,
+    })]
+    #[case::backtab_modifiers("ctrl backtab", KeyCombination {
+        code: KeyCode::Tab,
+        modifiers: KeyModifiers::CTRL | KeyModifiers::SHIFT,
+    })]
     fn test_parse_key_combination(
         #[case] input: &str,
         #[case] expected: impl Into<KeyCombination>,
@@ -497,7 +599,7 @@ mod tests {
     #[case::multiple_modifiers(
         "ctrl shift end",
         KeyCode::End,
-        KeyModifiers::CONTROL | KeyModifiers::SHIFT,
+        KeyModifiers::CTRL | KeyModifiers::SHIFT,
         true,
     )]
     #[case::missing_modifier(
@@ -538,7 +640,6 @@ mod tests {
             KeyCode::PageUp,
             KeyCode::PageDown,
             KeyCode::Tab,
-            KeyCode::BackTab,
             KeyCode::Delete,
             KeyCode::Insert,
             // Intentionally omitting Null (what is it??)
@@ -643,4 +744,46 @@ mod tests {
             "invalid type: integer `3`, expected a sequence",
         );
     }
+
+    /// Test that user-provided bindings take priority
+    #[rstest]
+    #[case::user_binding(
+        Action::Submit,
+        KeyCode::Char('w'),
+        KeyCode::Char('w'),
+        Some(Action::Submit)
+    )]
+    #[case::default_not_available(
+        Action::Submit,
+        KeyCode::Tab,
+        KeyCode::Enter,
+        None
+    )]
+    #[case::unbound(Action::Submit, vec![], KeyCode::Enter, None)]
+    fn test_user_bindings(
+        #[case] action: Action,
+        #[case] binding: impl Into<InputBinding>,
+        #[case] pressed: KeyCode,
+        #[case] expected: Option<Action>,
+    ) {
+        let engine = InputMap::new(indexmap! {action => binding.into()});
+        let event = KeyEvent {
+            code: pressed,
+            kind: KeyEventKind::Press,
+            modifiers: KeyModifiers::NONE,
+            state: KeyEventState::empty(),
+        };
+        let actual = engine
+            .iter()
+            .find_map(|(action, binding)| {
+                // Find the action mapped to the mocked event
+                if binding.matches(&event) {
+                    Some(action)
+                } else {
+                    None
+                }
+            })
+            .copied();
+        assert_eq!(actual, expected);
+    }
 }
