@@ -19,7 +19,11 @@ use slumber_core::{
     template::{Prompter, Template, TemplateChunk, TemplateContext},
 };
 use slumber_util::ResultTraced;
-use std::{path::Path, sync::Arc, time::Duration};
+use std::{
+    path::{Path, PathBuf},
+    sync::Arc,
+    time::Duration,
+};
 use tokio::task;
 use tracing::{error, info};
 
@@ -91,6 +95,12 @@ impl TuiState {
     /// Panic if we receive a message of a type that we expected the root TUI
     /// to handle.
     pub fn handle_message(&mut self, message: Message) -> anyhow::Result<()> {
+        // This message has to be handled specially because it involves a
+        // wholesale replacement of the state
+        if let Message::CollectionSelect(path) = message {
+            return self.select_collection(path);
+        }
+
         match &mut self.0 {
             TuiStateInner::Loaded(state) => state.handle_message(message),
             // Nothing to do in the error state
@@ -161,10 +171,35 @@ impl TuiState {
             }
         }
     }
+
+    /// Select a new collection file, replacing this state entirely
+    fn select_collection(&mut self, path: PathBuf) -> anyhow::Result<()> {
+        let collection_file = CollectionFile::new(Some(path))?;
+
+        // Reuse the existing DB connection and message channel. We clone
+        // because we can't move out of the old state until the new one has
+        // replaced it. These are cheap clones.
+        let (database, messages_tx) = match &self.0 {
+            TuiStateInner::Loaded(state) => {
+                (&state.database, &state.messages_tx)
+            }
+            TuiStateInner::Error {
+                database,
+                messages_tx,
+                ..
+            } => (database, messages_tx),
+        };
+        let database =
+            database.root().clone().into_collection(&collection_file)?;
+
+        *self = Self::load(database, collection_file, messages_tx.clone());
+        Ok(())
+    }
 }
 
 /// Inner enum for [TuiState] to avoid exposing the variant values
 #[derive(Debug)]
+#[expect(clippy::large_enum_variant)]
 enum TuiStateInner {
     /// TUI loaded successfully and is off and running
     Loaded(LoadedState),
@@ -224,7 +259,7 @@ impl LoadedState {
         let watcher =
             watch_collection(collection_file.path(), messages_tx.clone()).ok();
 
-        LoadedState {
+        let state = LoadedState {
             collection_file,
             collection,
             database,
@@ -232,7 +267,9 @@ impl LoadedState {
             request_store,
             view,
             _watcher: watcher,
-        }
+        };
+        state.update_collection_name();
+        state
     }
 
     /// Handle an incoming message. Any error here should trigger a subsequent
@@ -362,7 +399,10 @@ impl LoadedState {
 
             // All other messages are handled by the root TUI and should never
             // get here
-            Message::ClearTerminal | Message::Quit | Message::Draw => {
+            Message::CollectionSelect(_)
+            | Message::ClearTerminal
+            | Message::Quit
+            | Message::Draw => {
                 panic!(
                     "Unexpected message in TuiState; should have been handled \
                     by parent: {message:?}"
@@ -380,6 +420,7 @@ impl LoadedState {
     /// Reload state with a new collection
     fn reload_collection(&mut self, collection: Collection) {
         self.collection = collection.into();
+        self.update_collection_name();
 
         // Rebuild the whole view, because tons of things can change
         self.view = View::new(
@@ -387,11 +428,16 @@ impl LoadedState {
             self.database.clone(),
             self.messages_tx(),
         );
-        self.view.notify(format!(
-            "Reloaded collection from {}",
-            self.collection_file.path().to_string_lossy()
-        ));
+        self.view.notify("Reloaded collection");
     }
+
+    /// Update the collection name in the DB according to the loaded collection.
+    /// Call this whenever the collection is successfully loaded to ensure the
+    /// DB is kept up to date.
+    fn update_collection_name(&self) {
+        self.database.set_name(self.collection.name.as_deref());
+    }
+
     /// Render URL for a request, then copy it to the clipboard
     fn copy_request_url(&self) -> anyhow::Result<()> {
         let RequestConfig {
@@ -717,11 +763,18 @@ mod tests {
             TuiStateInner::Loaded(LoadedState { collection, ..}) => collection,
         );
         assert_eq!(collection.recipes.iter().count(), 0);
+        // Collection name should be set in the DB
+        assert_eq!(
+            harness.database.metadata().unwrap().name.as_deref(),
+            Some("Test")
+        );
 
         // Update the file, make sure it's reflected
         fs::write(
             file.path(),
             r#"
+name: Test Reloaded
+
 requests:
     test: !request
         method: "GET"
@@ -745,6 +798,11 @@ requests:
             TuiStateInner::Loaded(LoadedState { collection, ..}) => collection,
         );
         assert_eq!(collection.recipes.iter().count(), 1);
+        // Name was updatd too
+        assert_eq!(
+            harness.database.metadata().unwrap().name.as_deref(),
+            Some("Test Reloaded")
+        );
     }
 
     /// Test an error in the collection during initial load. Should shove us
@@ -829,11 +887,48 @@ requests:
         assert_eq!(collection.recipes.iter().count(), 0);
     }
 
+    /// Switch the selected request, which should rebuild the state entirely
+    #[rstest]
+    #[tokio::test]
+    async fn test_collection_switch(temp_dir: TempDir, harness: TestHarness) {
+        // Start with an empty collection
+        let file = collection_file(&temp_dir);
+
+        // Create a second collection
+        let other_collection = temp_dir.join("other_slumber.yml");
+        fs::write(
+            &other_collection,
+            r#"requests: {"r1": !request {"method": "GET", "url": "http://localhost"}}"#,
+        )
+        .unwrap();
+
+        let mut state = TuiState::load(
+            harness.database.clone(),
+            file.clone(),
+            harness.messages_tx().clone(),
+        );
+        // Make sure it loaded correctly
+        let collection = assert_matches!(
+            &state.0,
+            TuiStateInner::Loaded(LoadedState { collection, ..}) => collection,
+        );
+        assert_eq!(collection.recipes.iter().count(), 0);
+
+        state
+            .handle_message(Message::CollectionSelect(other_collection.clone()))
+            .unwrap();
+        let collection = assert_matches!(
+            &state.0,
+            TuiStateInner::Loaded(LoadedState { collection, ..}) => collection,
+        );
+        assert_eq!(collection.recipes.iter().count(), 1);
+    }
+
     /// Get a path to a collection file in a directory. The file will be created
     /// with an empty collection
     fn collection_file(directory: &Path) -> CollectionFile {
         let path = directory.join("slumber.yml");
-        fs::write(&path, "requests: {}").unwrap();
+        fs::write(&path, "name: Test").unwrap();
         CollectionFile::new(Some(path)).unwrap()
     }
 }
