@@ -355,10 +355,10 @@ mod tests {
             test_util::TestComponent,
         },
     };
-    use crossterm::event::KeyCode;
     use persisted::PersistedStore;
     use rstest::rstest;
     use slumber_util::Factory;
+    use terminput::KeyCode;
 
     #[rstest]
     fn test_edit_basic(harness: TestHarness, terminal: TestTerminal) {
@@ -486,8 +486,8 @@ mod tests {
 
         component
             .int()
-            .open_actions()
-            .send_keys([KeyCode::Enter, KeyCode::Char('!'), KeyCode::Enter])
+            .action("Edit Authentication")
+            .send_keys([KeyCode::Char('!'), KeyCode::Enter])
             .assert_empty();
         assert_eq!(
             component.data().override_value(),
