@@ -396,12 +396,12 @@ mod tests {
             test_util::TestComponent,
         },
     };
-    use crossterm::event::KeyCode;
     use persisted::PersistedStore;
     use rstest::rstest;
     use serde::Serialize;
     use slumber_core::collection::RecipeId;
     use slumber_util::Factory;
+    use terminput::KeyCode;
 
     #[derive(Debug, Serialize, persisted::PersistedKey)]
     #[persisted(Option<String>)]
@@ -571,8 +571,8 @@ mod tests {
 
         component
             .int_props(props_factory)
-            .open_actions()
-            .send_keys([KeyCode::Enter, KeyCode::Char('!'), KeyCode::Enter])
+            .action("Edit Row")
+            .send_keys([KeyCode::Char('!'), KeyCode::Enter])
             .assert_empty();
 
         let selected_row = component.data().select.data().selected().unwrap();
