@@ -58,7 +58,8 @@ impl<T> SelectItem<T> {
 pub struct SelectStateBuilder<Item, State> {
     items: Vec<SelectItem<Item>>,
     /// Store preselected value as an index, so we don't need to care about the
-    /// type of the value. Defaults to 0.
+    /// type of the value. Defaults to 0. If a filter is give, this will be
+    /// the index *after* the filter is applied.
     preselect_index: usize,
     subscribed_events: Vec<SelectStateEventType>,
     _state: PhantomData<State>,
@@ -119,6 +120,24 @@ impl<Item, State> SelectStateBuilder<Item, State> {
         }
     }
 
+    /// Apply a case-insensitive text filter to the list. Any item whose label
+    /// does not contain the text will be excluded from the list. During the
+    /// build, this should be called *before* and `preselect` functions to
+    /// ensure you don't select a value that will then be filtered out.
+    pub fn filter(mut self, filter: &str) -> Self
+    where
+        Item: ToString,
+    {
+        let filter = filter.trim();
+        if !filter.is_empty() {
+            let filter = filter.to_lowercase();
+            self.items.retain(|item| {
+                item.value.to_string().to_lowercase().contains(&filter)
+            });
+        }
+        self
+    }
+
     pub fn build(self) -> SelectState<Item, State>
     where
         State: SelectStateData,
@@ -286,18 +305,15 @@ impl<Item, State: SelectStateData> SelectState<Item, State> {
     /// event will *not* be emitted if no item is select, or the selected item
     /// is disabled.
     fn emit_for_selected(&self, event_fn: impl Fn(usize) -> SelectStateEvent) {
-        // unstable: if-let chain
-        // https://github.com/rust-lang/rust/pull/132833
-        match self.selected_index() {
+        if let Some(selected) = self.selected_index()
             // Don't send event for disabled items
-            Some(selected) if !self.items[selected].disabled => {
-                let event = event_fn(selected);
-                // Check if the parent subscribed to this event type
-                if self.is_subscribed(SelectStateEventType::from(&event)) {
-                    self.emitter.emit(event);
-                }
+            && !self.items[selected].disabled
+        {
+            let event = event_fn(selected);
+            // Check if the parent subscribed to this event type
+            if self.is_subscribed(SelectStateEventType::from(&event)) {
+                self.emitter.emit(event);
             }
-            _ => {}
         }
     }
 
@@ -308,7 +324,6 @@ impl<Item, State: SelectStateData> SelectState<Item, State> {
 
 impl<Item, State> Default for SelectState<Item, State>
 where
-    Item: PartialEq,
     State: SelectStateData,
 {
     fn default() -> Self {
@@ -490,13 +505,38 @@ mod tests {
             util::persistence::{DatabasePersistedStore, PersistedLazy},
         },
     };
-    use crossterm::event::KeyCode;
     use persisted::{PersistedKey, PersistedStore};
     use ratatui::widgets::List;
     use rstest::{fixture, rstest};
     use serde::Serialize;
     use slumber_core::collection::ProfileId;
     use slumber_util::assert_matches;
+    use terminput::KeyCode;
+
+    /// Apply a filter during build
+    #[rstest]
+    #[case::no_match("mango", None, &[], None)]
+    #[case::no_preselect("pl", None, &["apple", "APPLE"], Some(0))]
+    #[case::preselect_visible("pl", Some("APPLE"), &["apple", "APPLE"], Some(1))]
+    #[case::preselect_hidden("pl", Some("banana"), &["apple", "APPLE"], Some(0))]
+    fn test_filter(
+        #[case] filter: &str,
+        #[case] preselect: Option<&str>,
+        #[case] expected_visible: &[&str],
+        #[case] expected_selected: Option<usize>,
+    ) {
+        let items = ["apple", "APPLE", "banana"];
+        let select: SelectState<&str, ListState> =
+            SelectState::builder(items.into_iter().collect())
+                .filter(filter)
+                .preselect_opt(preselect.as_ref())
+                .build();
+        assert_eq!(
+            select.items().copied().collect::<Vec<_>>(),
+            expected_visible
+        );
+        assert_eq!(select.selected_index(), expected_selected);
+    }
 
     /// Test going up and down in the list
     #[rstest]
